/*******************************************************************************
 * $Id: IntervalList.h,v 1.6 2004/03/01 02:40:08 darling Exp $
 * This file is copyright 2002-2004 Aaron Darling.  All rights reserved.
 * Please see the file called COPYING for licensing, copying, and modification
 * rights.  Redistribution of this file, in whole or in part is prohibited
 * without express permission.
 ******************************************************************************/

#ifndef _IntervalList_h_
#define _IntervalList_h_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <list>
#include "libMems/SortedMerList.h"
#include "libGenome/gnSequence.h"
#include "libMems/Interval.h"

/**
 * This class represents a set Intervals, each of which is a collinear aligned region
 * There are functions to read and write an IntervalList.
 * @see Interval
 */
class IntervalList : public vector< Interval > {
public:
	IntervalList(){};
	IntervalList( const IntervalList& ml );
	IntervalList& operator=( const IntervalList& ml );
	~IntervalList(){};
	
	/**
	 * Deletes the objects associated
	 * with this IntervalList.
	 */
	void Clear();

	/**
	 * Reads a IntervalList from an input stream
	 * Sequence and SML file names are read into the seq_filename
	 * and sml_filename vectors, but the actual files are not
	 * opened.  The calling function should load them after
	 * using this method.
	 * @param match_stream The input stream to read from
	 */
	void ReadList( istream& match_stream );

	/**
	 *  Writes a IntervalList to the designated output stream
	 * @param match_stream The outptu stream to write to
	 */
	void WriteList( ostream& match_stream ) const;
	
	/**
	 *  Writes a gapped alignment of sequences to the output stream
	 */
	void WriteAlignedSequences(ostream& match_file) const;
	
	/**
	 *	Writes a gapped alignment of sequences in a standard format
	 */
	void WriteStandardAlignment( ostream& out_file ) const;
	
	/**
	 * Reads in a set of intervals that are in xmfa (eXtended MultiFastA) format
	 */
	void ReadStandardAlignment( istream& in_stream );
	
	vector<string> seq_filename;	/**< The names of files associated with the sequences used by this alignment */
	vector<gnSequence*> seq_table;	/**< The actual sequences used in this alignment */

protected:

};


#endif	//_IntervalList_h_
